/*-----------------------------------------
* chicken_mobile.h
*------------------------------------------*/

// Port definitions: MP3
#define PORT_MP3 PORTB
#define DDR_MP3 DDRB
#define PORTBIT_MP3_ON 0
#define PORTBIT_MP3_SONG 1

// Port definitions: Power
#define PORT_POWER PORTB
#define DDR_POWER DDRB
#define PORTBIT_POWER 2

// Port definitions Hatch control
#define PORT_HATCH PORTD
#define DDR_HATCH DDRD
#define PORTBIT_HATCH 7

// job flags
#define JF_ONCE_PER_SECOND 0
#define JF_DISPLAY_LIGHT_ON 1
#define JF_DISPLAY_LIGHT_OFF 2
#define JF_REFRESH_DISPLAY 3

// scheduler flags
#define SF_SYNC_TIME 0
#define SF_READ_EXT_TEMP 1
#define SF_READ_INT_TEMP 2
#define SF_CHECK_BATT_STATUS 3
#define SF_CHECK_LIGHT_STATUS 4
#define SF_UPDATE_LED_STATUS 5
#define SF_UPDATE_MP3_STATUS 6
#define SF_CHECK_HATCH 7

// I2C addresses
#define RTC_ADDR 0x68

// status definitions
#define OFF 0
#define ON 1
#define LIGHT_LEVEL_HIGH 1
#define LIGHT_LEVEL_LOW 0
#define BATT_LEVEL_HIGH 2
#define BATT_LEVEL_CRIT 1
#define BATT_LEVEL_LOW 0
#define LED_OFF 0		// LED is off, check for time window and ambient light level
#define LED_ON 1		// LED is on, check for duration and time window end
#define LED_ON_CRIT 2	// LED is on, but reduced due to low battery voltage
#define LED_DIM 3		// LED is in ramp down mode
#define LED_FIN 4		// LED is finished for the day, reset at midnight
#define LED_AUTO 0
#define LED_MAN 1
#define MP3_OFF 0
#define MP3_ON 1
#define MP3_TW_OFF 0
#define MP3_TW_ON 1 
#define MP3_AUTO 0
#define MP3_MAN 1
#define HATCH_AUTO 0
#define HATCH_MAN 1


// default values
#define DEFAULT_DISPLAY_LIGHT_LEVEL 8
#define DEFAULT_DISPLAY_TIMEOUT 120
#define DEFAULT_BATT_THESHOLD_HIGH 1250
#define DEFAULT_BATT_THESHOLD_CRIT 1200
#define DEFAULT_BATT_THESHOLD_LOW 1100
#define DEFAULT_AMB_LIGHT_THRESHOLD 500
#define DEFAULT_LED_DURATION 180
#define DEFAULT_LED_TW_ON_HOUR 16
#define DEFAULT_LED_TW_ON_MIN 0
#define DEFAULT_LED_TW_OFF_HOUR 21
#define DEFAULT_LED_TW_OFF_MIN 0
#define DEFAULT_LED_MAX 30
#define DEFAULT_AMB_LIGHT_MOV_AVG_DUR 5
#define DEFAULT_MP3_TW1_ON_HOUR 6
#define DEFAULT_MP3_TW1_ON_MIN 0
#define DEFAULT_MP3_TW1_OFF_HOUR 11
#define DEFAULT_MP3_TW1_OFF_MIN 0
#define DEFAULT_MP3_TW2_ON_HOUR 16
#define DEFAULT_MP3_TW2_ON_MIN 0
#define DEFAULT_MP3_TW2_OFF_HOUR 21
#define DEFAULT_MP3_TW2_OFF_MIN 0
#define DEFAULT_MP3_DURATION 7
#define DEFAULT_HATCH_HOUR 6
#define DEFAULT_HATCH_MIN 30
#define DEFAULT_HATCH_DUR 300

// max constants
#define MAX_LED_LEVEL 30
#define MAX_DISPLAY_LIGHT 10
#define MAX_AMB_LIGHT_MOV_AVG_DUR 20
#define MAX_MP3_SONGS 10
#define MAX_TEMP 1500					// values above 100.0 indicate error status
#define MAX_BATT 2500

// buffer size
#define TEMP_BUFFER_SIZE 175	
#define LIGHT_START_BUFFER_SIZE 10

// message box
#define MESSAGE_BOX_X 17
#define MESSAGE_BOX_Y 20 

// data type definitions
#ifndef cmh_typedef_
#define cmh_typedef_

typedef struct {
	uint8_t hour;
	uint8_t min;
} light_start_time_t;

typedef struct {
	uint8_t year;
	uint8_t month;
	uint8_t day;
	uint8_t day_of_the_week;
	uint8_t hour;
	uint8_t min;
	uint8_t sec;
} datetime_t;

typedef struct {
	uint16_t value;
	const char *string;
	uint16_t min;
	uint16_t max;
	uint16_t step;
	uint8_t decimal_places;
} edit_frame_t;

typedef struct {
	volatile uint16_t vcc;			// current battery voltage
	uint8_t vcc_status;				// current status: high, critical, low
	uint16_t threshold_high;		// threshold
	uint16_t threshold_crit;
	uint16_t threshold_low;
} batt_status_t;

typedef struct {
	volatile uint16_t sens1;
	volatile uint16_t sens2;
	uint8_t sens_status;			// LIGHT_LEVEL_HIGH or LIGHT_LEVEL_LOW
	uint16_t light_threshold;
	uint16_t light_mov_avg;			// current moving average
	uint8_t light_mov_avg_dur;		// duration [min] to calculate the moving average
} amb_light_status_t;

typedef struct {
	int16_t temp_ext;				// external temperature
	uint8_t err_flag_ext;
	int16_t temp_int;				// internal temperature
	uint8_t err_flag_int;
} temp_status_t;

typedef struct {
	uint8_t light_level;			// display back light level
	volatile uint16_t timeout_cnt;	// seconds till timeout
	uint16_t timeout;
} display_status_t;

typedef struct {
	uint8_t led_level;				// current level: 0 ... 10
	uint8_t led_status;				// values: LED_OFF, LED_ON, LED_ON_CRIT, LED_DIM
	uint8_t led_tw_on_hour;			// LED time window start - hour
	uint8_t led_tw_on_min;			// LED time window start - minute
	uint8_t led_tw_off_hour;		// LED time window end - hour
	uint8_t led_tw_off_min;			// LED time window end - min
	uint16_t led_duration;			// running time for LED (minutes)
	uint16_t led_duration_cnt;		// if status on: remaining minutes till dim phase
	uint8_t led_max;				// maximum LED light level
	uint8_t led_mode;				// values: LED_AUTO, LED_MAN
} LED_status_t;

typedef struct {
	uint8_t mp3_mode;				// MP3_MAN or MP3_AUTO
	uint8_t mp3_tw1_on_hour;		
	uint8_t mp3_tw1_on_min;
	uint8_t mp3_tw1_off_hour;
	uint8_t mp3_tw1_off_min;
	uint8_t mp3_tw2_on_hour;		
	uint8_t mp3_tw2_on_min;
	uint8_t mp3_tw2_off_hour;
	uint8_t mp3_tw2_off_min;
	uint8_t song_nr;				// sequence number for the current play
	uint8_t song_cnt;				// internal song counter
	uint8_t mp3_status;				// MP3_OFF or MP3_ON
	uint8_t mp3_duration;			// length of each play [min]
	uint8_t mp3_duration_cnt;		// min-counter for current play	
	uint8_t mp3_tw1_status;
	uint8_t mp3_tw2_status;
	uint8_t mp3_seq;				// sequencer
} mp3_status_t;

typedef struct {
	uint8_t hatch_mode;				// HATCH_MAN or HATCH_AUTO
	uint8_t hatch_hour;
	uint8_t hatch_min;
	uint16_t hatch_dur;				// time (secs) to keep the signal high 
	uint16_t hatch_cnt;				// if open then remaining secs till closure
} hatch_control_t;

#endif



