/*
* cmh_util.c
* Collection of utility functions for Chicken Mobile Home
*/

#ifndef cmh_f_cpu_
#define cmh_f_cpu_

#define F_CPU 8000000L

#endif

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "chicken_mobile.h"
#include "i2cmaster.h"
#include "glcd_routines.h"
#include "display_routines.h"
#include "ee_routines.h"

/*----------------------------------------------------------------------------------------*/
uint8_t bcd_to_hex(const uint8_t bcd) {
	return((bcd / 16 * 10) + (bcd & 0x0f));
}

/*----------------------------------------------------------------------------------------*/
uint8_t hex_to_bcd(const uint8_t hex) {
	return((hex / 10 * 16) + (hex % 10));
}

/*----------------------------------------------------------------------------------------*/
void show_2dig_dec(uint8_t value) {
	glcd_draw_char('0' + value / 10);
	glcd_draw_char('0' + value % 10);
}

/* time_after -----------------------------------------------------------------------------------
* Compares 2 times.
* Returns 1 in case time1 is after or equal to time2.
* Returns 0 in case time1 is before time2
*/
uint8_t time_after(const uint8_t time1_hour, const uint8_t time1_min, const uint8_t time2_hour, const uint8_t time2_min) {
	if (time1_hour < time2_hour) return(0);
	if (time1_hour > time2_hour) return(1);
	if (time1_min < time2_min) return(0);
	return(1);
}

/* decrease_day ---------------------------------------------------------------------------------
*/
void decrease_day(uint8_t *day, uint8_t *month) {
	extern datetime_t date_time;
	
	if (*day > 1) {
		--*day;
		return;
	};
	
	switch (*month) {
		case 1:
			*day = 31;
			break;
		case 2:
			*day = 31;
			break;
		case 3:
			if ((date_time.year % 4) == 0) *day = 29;
			else *day = 28;
			break;
		case 4:
			*day = 31;
			break;
		case 5:
			*day = 30;
			break;
		case 6:
			*day = 31;
			break;
		case 7:
			*day = 30;
			break;
		case 8:
			*day = 31;
			break;
		case 9:
			*day = 31;
			break;
		case 10:
			*day = 30;
			break;
		case 11:
			*day = 31;
			break;
		case 12:
			*day = 30;
	};
	if (*month > 1)	--*month;
	else *month = 12;
}

/* check_summer_winter_time --------------------------------------------------------------*
* Returns 1 in case of summer time and 0 in case of winter time
*/

uint8_t check_summer_winter_time() {
	extern datetime_t date_time;
	const uint8_t tzHours = 1;					// 0 = UGT, 1 = MEZ 

	if ((date_time.month < 3) || (date_time.month > 10)) return(0);				// no summertime from Nov to Feb
	if ((date_time.month > 3) && (date_time.month < 10)) return(1);				// always summertime from Apr to Sep
	if (date_time.month == 3) {
		if ((date_time.hour + 24 * date_time.day) >= (1 + tzHours + 24 * (31 - (5 * date_time.year / 4 + 4) % 7)))
			return(1);
		else
			return(0);
	};
	if (date_time.month == 10) {
		if ((date_time.hour + 24 * date_time.day) < (1 + tzHours + 24 * (31 - (5 * date_time.year / 4 + 1) % 7)))
			return(1);
		else
			return(0);
	};
	return(255);																// something went wrong ...
}

/* ---------------------------------------------------------------------------------------*/
void get_date_time(void) {
	extern datetime_t date_time;
	extern light_start_time_t light_start_time[];
	extern uint8_t light_start_pnt;
	
	i2c_start_wait(2 * RTC_ADDR + I2C_WRITE);
	i2c_write(0);
	i2c_start_wait(2 * RTC_ADDR + I2C_READ);
	date_time.sec = bcd_to_hex(i2c_readAck());
	date_time.min = bcd_to_hex(i2c_readAck());
	date_time.hour = bcd_to_hex(i2c_readAck());
	date_time.day_of_the_week = bcd_to_hex(i2c_readAck());
	date_time.day = bcd_to_hex(i2c_readAck());
	date_time.month = bcd_to_hex(i2c_readAck());
	date_time.year = bcd_to_hex(i2c_readNak());
	i2c_stop();
	
	if (check_summer_winter_time() > 0) {					// check for summer time
		++date_time.hour;									// add 1 hour
		if (date_time.hour >= 24) {
			date_time.hour = 0;
			++date_time.day;								// add 1 day
			if ((date_time.month == 4) || (date_time.month == 6) ||	(date_time.month == 9)) {		
				if (date_time.day > 30) {					// Apr, Jun, Sep
					date_time.day = 1;
					++date_time.month;
				};
			} else {										// Mar, May, Jul, Aug, Oct
				if (date_time.day > 31) {
					date_time.day = 1;
					++date_time.month;
				};
			};
		};
	};

	// in case a new day is starting ... 
	if ((date_time.min == 0) && (date_time.hour == 0)) {
		if (++light_start_pnt >= LIGHT_START_BUFFER_SIZE)	// advance the day pointer
			light_start_pnt = 0;
		light_start_time[light_start_pnt].hour = 0xff;		// clear time for the running day
		light_start_time[light_start_pnt].min = 0xff;
	};
}

/* ---------------------------------------------------------------------------------------*/
void set_date_time(void) {
	extern datetime_t date_time;

	if (check_summer_winter_time() > 0) { 		// deduct 1 hour
		if (date_time.hour == 0) {
			date_time.hour = 23;
			if (date_time.day == 1) {
				--date_time.month; 
				if ((date_time.month == 4) || (date_time.month == 6) ||	(date_time.month == 9))
					date_time.day = 30;
				else
					date_time.day = 31;
			};
		} else {
			--date_time.hour;
		};
	};

	i2c_start_wait(2 * RTC_ADDR + I2C_WRITE);
	i2c_write(0);
	i2c_write(0);		// seconds
	i2c_write(hex_to_bcd(date_time.min));
	i2c_write(hex_to_bcd(date_time.hour));
	i2c_write(0);		// day of week - not used
	i2c_write(hex_to_bcd(date_time.day));
	i2c_write(hex_to_bcd(date_time.month));
	i2c_write(hex_to_bcd(date_time.year));
	i2c_stop();
	
	_delay_ms(10);
	get_date_time();
};

/* ---------------------------------------------------------------------------------------*/
uint16_t DS3231_get_temperature(void) {
	uint8_t low_byte;
	uint16_t temp;

	i2c_start_wait(2 * RTC_ADDR + I2C_WRITE);
	i2c_write(0x11);
	i2c_start_wait(2 * RTC_ADDR + I2C_READ);
	temp = i2c_readAck() * 100;
	low_byte = i2c_readNak();
	i2c_stop();
	temp = temp + (low_byte >> 6) * 25;
	return(temp);
}

/* set_display_light ---------------------------------------------------------------------*
* Sets the LCD display background light
* Input: light_level - ranges from 0 (off) to 10 (100%)
*/
void set_display_light(uint8_t new_light_level) {
	extern display_status_t display_status;
	
	if (new_light_level > MAX_DISPLAY_LIGHT) new_light_level = MAX_DISPLAY_LIGHT;
	glcd_set_light(new_light_level);
	if (new_light_level != display_status.light_level) {
		display_status.light_level = new_light_level;
		ee_write_display_status();
	};
}

/* check_batt_status --------------------------------------------------------------------
* Checks the battery voltage and sets the batt_status accordingly
* Sets global batt_status
*/
void check_batt_status(void) {
	extern batt_status_t batt_status;
	
	if (batt_status.vcc >= batt_status.threshold_high) {
		batt_status.vcc_status = BATT_LEVEL_HIGH;
		return;
	};
	if (batt_status.vcc <= batt_status.threshold_low) {
		batt_status.vcc_status = BATT_LEVEL_LOW;
		return;
	};
	if (batt_status.vcc <= batt_status.threshold_crit) {
		batt_status.vcc_status = BATT_LEVEL_CRIT;
	};
	return;
}

/* check_amb_light_status ---------------------------------------------------------------------
* Checks the light levels and sets the status accordingly
* Sets global light_status
*/
void check_amb_light_status(void) {
	extern amb_light_status_t amb_light_status;
	static uint16_t amb_light_buffer[MAX_AMB_LIGHT_MOV_AVG_DUR];
	static uint8_t amb_light_buffer_pnt = 0, init_flag = 255;		
	uint16_t sum;
	uint8_t i, pnt;
	
	// initiate buffer - only running at the first function call
	if (init_flag == 255) init_flag = amb_light_status.light_mov_avg_dur;
	
	// calculate maximum and save value to buffer
	if (amb_light_status.sens1 > amb_light_status.sens2)
		amb_light_buffer[amb_light_buffer_pnt] = amb_light_status.sens1;
	else
		amb_light_buffer[amb_light_buffer_pnt] = amb_light_status.sens2;

	// calculate moving average
	if (init_flag > 0) {									// during initiation: just capture data
		amb_light_status.light_mov_avg = 2000;				// set value to "invalid"
		amb_light_status.sens_status = LIGHT_LEVEL_HIGH;	// set status to default
		--init_flag;
	} else {						// after initiation: calculate moving average
		sum = 0;
		pnt = amb_light_buffer_pnt;
		for (i = 0; i < amb_light_status.light_mov_avg_dur; ++i) {
			sum += amb_light_buffer[pnt];
			if (pnt == 0) pnt = amb_light_status.light_mov_avg_dur - 1;
			else --pnt;
		};
		amb_light_status.light_mov_avg = sum / amb_light_status.light_mov_avg_dur;
		// set status flag
		if (amb_light_status.light_mov_avg >= amb_light_status.light_threshold)
			amb_light_status.sens_status = LIGHT_LEVEL_HIGH;
		else 
			amb_light_status.sens_status = LIGHT_LEVEL_LOW;
	};
	
	// advance pointer in preparation for the next round
	if (++amb_light_buffer_pnt == amb_light_status.light_mov_avg_dur)
		amb_light_buffer_pnt = 0;
};

		
/* set_LED_level ---------------------------------------------------------------------------
* Sets the LED level on PORTD, pin 6
* Input: uint8_t LED_level: 0 ... 10
* Global variables: LED_status
*/
void set_LED_level(uint8_t LED_level) {
	extern LED_status_t LED_status;
	extern const uint8_t LED_light_table[];
	
	if (LED_level > MAX_LED_LEVEL) LED_level = MAX_LED_LEVEL;
	LED_status.led_level = LED_level;
	
	// full power on
	if (LED_level == MAX_LED_LEVEL) {		// LED is full power on
		TCCR0A = 0;							// stop PWM
		PORTD |= (1 << 6);
		return;		
	};
	
	// completely off
	if (LED_level == 0) {					// LED is completely off
		TCCR0A = 0;							// stop PWM
		PORTD &= ~(1 << 6);
		return;
	};

	// anything in-between requires PWM 
	OCR0A = pgm_read_byte(LED_light_table + LED_level);		// set PWM 
	TCCR0A |= (1 << COM0A1);								// set none-inverting mode
	TCCR0A |= (1 << WGM01) | (1 << WGM00);					// set fast PWM Mode
	TCCR0B |= (1 << CS01) | (1 << CS00);					// set prescaler to 64 and start PWM
	return;	
}


/* update_LED_status ---------------------------------------------------------------------------
* Sets the LED status according to time window and ambient light. 
* This function needs to be called once per minute  
* Input: none
* Output: none
* Global variables: LED_status, date_time
*/
void update_LED_status(void) {
	extern LED_status_t LED_status;
	extern datetime_t date_time;
	extern amb_light_status_t amb_light_status;
	extern batt_status_t batt_status;
	extern light_start_time_t light_start_time[];
	extern uint8_t light_start_pnt;
	
	if (LED_status.led_mode == LED_MAN) return;				// in case of manual: don't change anything
	
	switch (LED_status.led_status) {
		case LED_OFF:
			// check start of time window. If current time is after time window start, check light level
			if (time_after(date_time.hour, date_time.min, LED_status.led_tw_on_hour, LED_status.led_tw_on_min) > 0) { 
				if (amb_light_status.sens_status == LIGHT_LEVEL_LOW) {
					set_LED_level(LED_status.led_max);						// switch on LED
					LED_status.led_duration_cnt = LED_status.led_duration;	// load duration counter
					LED_status.led_status = LED_ON;							// change to ON mode
					light_start_time[light_start_pnt].hour = date_time.hour;	// capture LED start time 
					light_start_time[light_start_pnt].min = date_time.min;
				};
			};
			break;			
		case LED_ON:
			// check battery level. If battery status is critical, reduce LED level by factor 2
			if (batt_status.vcc_status == BATT_LEVEL_CRIT) {
				set_LED_level(LED_status.led_level / 2);		// reduce LED level by factor 2
				LED_status.led_status = LED_ON_CRIT;			// change status, no further checking for critical battery status
			};
		case LED_ON_CRIT:
			// check battery level. If battery status is low, switch off LED
			if (batt_status.vcc_status == BATT_LEVEL_LOW) {
				set_LED_level(0);								// switch off LED
				LED_status.led_status = LED_FIN;				// done
				break;
			};			
			// check end of time window. If current time is after end of time window, set remaining duration to 0
			if (time_after(date_time.hour, date_time.min, LED_status.led_tw_off_hour, LED_status.led_tw_off_min) > 0) {
				LED_status.led_duration_cnt = 0;				// if current time after time window end, shut down LED
			} else {
				--LED_status.led_duration_cnt;					// otherwise reduce remaining LED duration
			};
			if (LED_status.led_duration_cnt == 0) {
				LED_status.led_status = LED_DIM;				// if duration has expired, switch to DIM mode
			};
			break;
			
		case LED_DIM:
			// check battery level. If battery status is low, switch off LED
			if (batt_status.vcc_status == BATT_LEVEL_LOW) {
				set_LED_level(0);								// switch off LED
				LED_status.led_status = LED_FIN;				// done
				break;
			};
			// reduce LED level, once per minute
			if (LED_status.led_level > 0) {
				set_LED_level(LED_status.led_level - 1);		// in DIM mode, decrease LED level once per minute;
			} else {
				LED_status.led_status = LED_FIN;				// if dim steps have expired, change to FIN mode
			};
			break;	
			
		case LED_FIN:											// reset LED status to OFF at midnight
			if (date_time.hour == 0) LED_status.led_status = LED_OFF;
			break;
	};
}

/* mp3_sequencer ---------------------------------------------------------------------------------------------
* Manages the song selection for MP3 player
* This function needs to be called once per second
*/
void mp3_sequencer(void) {
	extern mp3_status_t mp3_status;
	extern uint8_t mp3_trigger;
	
	if (mp3_status.mp3_seq == 0) return;				// same as above, nothing to do
	
	// case: 1st song
	switch (mp3_status.mp3_seq) {
		case 7:
			mp3_status.song_cnt = mp3_status.song_nr;	// initiate internal song counter
			--mp3_status.mp3_seq;						// wait 1 second
			break;
		case 6:
			--mp3_status.mp3_seq;						// wait 1 second
			break;			
		case 5:
			PORT_MP3 |= (1 << PORTBIT_MP3_SONG);		// set MP3 song to high
			--mp3_status.mp3_seq;						// wait 1 second
			break;
		case 4:
			--mp3_status.mp3_seq;						// wait 1 second
			break;
		case 3:
			--mp3_status.mp3_seq;						// wait 1 second
			break;
		case 2:
			PORT_MP3 &= ~(1 << PORTBIT_MP3_SONG);		// set MP3 song to low
			if (mp3_status.song_cnt == 1) {
				mp3_status.mp3_seq = 0;					// done
			} else {
				mp3_status.mp3_seq = 1;					// otherwise switch to short impulse triggers
			};
			break;
		case 1:
			PORT_MP3 |= (1 << PORTBIT_MP3_SONG);		// set MP3 song to high - will be reset by IRQ
			mp3_trigger = 6;							// initiate trigger to 6 * 50 msec = 300 msec
			if (--mp3_status.song_cnt == 1)				// reduce remaining number of songs
				mp3_status.mp3_seq = 0;					// if this was the last song, then we are done
			break;			
	}
}

/* switch_mp3_on ---------------------------------------------------------------------------------------------
* Switches the MP3 player on and advances to the next song
* Called by "check_mp3_switch_on()" and test mode
*/
void switch_mp3_on(void) {
	extern mp3_status_t mp3_status;

	mp3_status.mp3_status = MP3_ON;							// change status to active
	PORT_MP3 |= (1 << PORTBIT_MP3_ON);						// enable MP3 player
	mp3_status.mp3_duration_cnt = mp3_status.mp3_duration;	// initiate duration counter
	mp3_status.mp3_seq = 7;									// initiate sequencer 
}

/* switch_mp3_off --------------------------------------------------------------------------------------------
* Switches the MP3 player off
* Called by "update_mp3_status()" and test mode
*/
void switch_mp3_off(void) {
	extern mp3_status_t mp3_status;

	PORT_MP3 &= ~(1 << PORTBIT_MP3_ON);					// disable MP3 player
	mp3_status.mp3_status = MP3_OFF;					// change status to inactive
} 

/* check_mp3_switch_on ---------------------------------------------------------------------------------------
* Checks whether start conditions for the MP3 player are met, and switch MP3 player on if that is the case
* Local function to support "update_mp3_status()"
*/
static void check_mp3_switch_on(void) {
	extern mp3_status_t mp3_status;
	extern datetime_t date_time;
	extern batt_status_t batt_status;
	
	if ((date_time.min == 0) && (batt_status.vcc_status != BATT_LEVEL_LOW)) {
		switch_mp3_on();
	};
	if ((date_time.min == 30) && (batt_status.vcc_status == BATT_LEVEL_HIGH)) {
		switch_mp3_on();
	};
};

/* update_mp3_status -----------------------------------------------------------------------------------------
* Manages the MP3 player.
* This function needs to be called once per minute
*/

void update_mp3_status(void) {
	extern mp3_status_t mp3_status;
	extern batt_status_t batt_status;
	extern datetime_t date_time;
	
	// case: MP3 control is manual- nothing to do
	if (mp3_status.mp3_mode == MP3_MAN) return;
	
	// case: MP3 is active
	if (mp3_status.mp3_status == MP3_ON) {
		--mp3_status.mp3_duration_cnt;
		if (mp3_status.mp3_duration_cnt == 0) {
			switch_mp3_off();												// switch off
			if (mp3_status.song_nr < MAX_MP3_SONGS) ++mp3_status.song_nr;	// prepare for next song
			return;
		};
	};
	
	// case: MP3 player is off, but time window 1 is active
	if (mp3_status.mp3_tw1_status == MP3_TW_ON) {
		// checking whether we are still within time window
		if (time_after(date_time.hour, date_time.min, mp3_status.mp3_tw1_off_hour, mp3_status.mp3_tw1_off_min)) {
			mp3_status.mp3_tw1_status = MP3_TW_OFF;			// change status to "outside time window"
			mp3_status.song_nr = 1;							// reset sequence number
		} else {
			check_mp3_switch_on();
		};
		return;
	};

	// case: MP3 player is off, but time window 2 is active
	if (mp3_status.mp3_tw2_status == MP3_TW_ON) {
		// checking whether we are still within time window
		if (time_after(date_time.hour, date_time.min, mp3_status.mp3_tw2_off_hour, mp3_status.mp3_tw2_off_min)) {
			mp3_status.mp3_tw2_status = MP3_TW_OFF;			// change status to "outside time window"
			mp3_status.song_nr = 1;							// reset sequence number
		} else {
			check_mp3_switch_on();
		};
		return;
	};
			
	// remaining case: player is off, and we are outside both time windows	
	// checking if current time is inside time window 1
	if (time_after(date_time.hour, date_time.min, mp3_status.mp3_tw1_on_hour, mp3_status.mp3_tw1_on_min) &&
		time_after(mp3_status.mp3_tw1_off_hour, mp3_status.mp3_tw1_off_min, date_time.hour, date_time.min)) {
			mp3_status.mp3_tw1_status = MP3_TW_ON;
	} else {
		// checking if current time is inside time window 2
		if (time_after(date_time.hour, date_time.min, mp3_status.mp3_tw2_on_hour, mp3_status.mp3_tw2_on_min) &&
			time_after(mp3_status.mp3_tw2_off_hour, mp3_status.mp3_tw2_off_min, date_time.hour, date_time.min)) {
				mp3_status.mp3_tw2_status = MP3_TW_ON;
		};
	};
	if ((mp3_status.mp3_tw1_status == MP3_TW_ON) || (mp3_status.mp3_tw2_status == MP3_TW_ON)) {
		check_mp3_switch_on();
	};
};

/* hatch_switch_on -------------------------------------------------------------------------------------------
* Switches the hatch output to high.
*/

void hatch_switch_on(void) {
	extern hatch_control_t hatch_control;
	
	PORT_HATCH &= ~(1 << PORTBIT_HATCH);					// set output to low
	hatch_control.hatch_cnt = hatch_control.hatch_dur;		// start timer
}

/* hatch_switch_off ------------------------------------------------------------------------------------------
* Switches the hatch output to low. 
*/

void hatch_switch_off(void) {
	extern hatch_control_t hatch_control;
	
	PORT_HATCH |= (1 << PORTBIT_HATCH);						// set output to high
	hatch_control.hatch_cnt = 0;							// stop timer
}

/* update_hatch_status ---------------------------------------------------------------------------------------
* Checks whether it is time to switch on the hatch
*/
void update_hatch_status(void) {
	extern hatch_control_t hatch_control;
	extern datetime_t date_time;
	
	if (hatch_control.hatch_mode == HATCH_AUTO) {
		if ((date_time.hour == hatch_control.hatch_hour) && (date_time.min == hatch_control.hatch_min))
		hatch_switch_on();
	};
}

/* hatch_sequencer -------------------------------------------------------------------------------------------
* Manages the hatch output. Switches to low at the end of duration period.
* This function needs to be called once per second
*/

void hatch_sequencer(void) {
	extern hatch_control_t hatch_control;
	
	if (hatch_control.hatch_cnt > 0) {
		--hatch_control.hatch_cnt;
		if (hatch_control.hatch_cnt == 0) {
			hatch_switch_off();
		};
	};
}

/* log_temperature -------------------------------------------------------------------------------------------
* Captures the temperature values in log data array
* This function needs to be called once per minute
*/
void log_temperature(void) {
	static uint8_t ext_cnt = 0, int_cnt = 0, batt_cnt = 0;
	static int16_t ext_avg = 0, int_avg = 0;
	static int16_t batt_avg = 0;
	extern temp_status_t temp_status;
	extern batt_status_t batt_status;
	extern datetime_t date_time;
	extern int16_t temp_buffer_ext[];
	extern int16_t temp_buffer_int[];
	extern uint16_t battery_buffer[];
	extern uint8_t temp_buffer_pnt;
	
	// capture external temperature
	if (temp_status.err_flag_ext == 0) {
		ext_avg += temp_status.temp_ext;
		++ext_cnt;
	};
	// capture internal temperature
	if (temp_status.err_flag_int == 0) {
		int_avg += temp_status.temp_int;
		++int_cnt;
	};
	// capture battery voltage
	batt_avg += batt_status.vcc;
	++batt_cnt;
		
	// check whether result needs to be logged (every 10 minutes)
	if (date_time.min % 10 == 0) {
		// capture log
		if (ext_cnt == 0)
			temp_buffer_ext[temp_buffer_pnt] = MAX_TEMP;
		else
			temp_buffer_ext[temp_buffer_pnt] = ext_avg / ext_cnt;
		if (int_cnt == 0)
			temp_buffer_int[temp_buffer_pnt] = MAX_TEMP;
		else
			temp_buffer_int[temp_buffer_pnt] = int_avg / int_cnt;
		battery_buffer[temp_buffer_pnt] = batt_avg / batt_cnt;
		// reset data
		ext_cnt = 0;
		int_cnt = 0;
		ext_avg = 0;
		int_avg = 0;
		batt_cnt = 0;
		batt_avg = 0;
		if (++temp_buffer_pnt >= TEMP_BUFFER_SIZE)
			temp_buffer_pnt = 0;
	};
}


