/*
* strings_pgm.c
*/

#include <avr/pgmspace.h>
#include "chicken_mobile.h"

// Strings to be stored in program memory and used f0r various purposes
const char str_title[] PROGMEM = "Chicken Mobile Home";
const char str_month_list[] PROGMEM = "JanFebMarAprMayJunJulAugSepOctNovDec";

// Menus
const char str_menu_cockpit[] PROGMEM = "-- Cockpit --\0-\0-\0test\0setup\0";
const char str_menu_setup_1[] PROGMEM = "-- Setup 1 --\0back\0-\0display\0date/time\0";
const char str_menu_setup_2[] PROGMEM = "-- Setup 2 --\0back\0-\0LED TW\0LED D/M\0";
const char str_menu_setup_3[] PROGMEM = "-- Setup 3 --\0back\0Amb Light\0-\0-\0-\0";
const char str_menu_setup_4[] PROGMEM = "-- Setup 4 --\0back\0hatch ctr\0-\0-\0-\0";
const char str_menu_test_output[] PROGMEM = "- Test Output -\0back\0Hatch\0MP3 On\0song #\0";

// edit function strings. Structure: heading title, menu 1, menu 2, menu 3, menu 4, label
const char str_edit_timeout[] PROGMEM = "- Edit Timeout -\0Cancel\0\0\0Okay\0Timeout [min]";
const char str_edit_backlight[] PROGMEM = "- Edit Backlight -\0Cancel\0\0\0Okay\0Backlight [0-10]";
const char str_edit_year[] PROGMEM = "-- Year --\0Cancel\0\0\0Okay\0Year";
const char str_edit_month[] PROGMEM = "-- Month --\0Cancel\0\0\0Okay\0Month";
const char str_edit_day[] PROGMEM = "-- Day --\0Cancel\0-5\0+5\0Okay\0Day";
const char str_edit_hour[] PROGMEM = "-- Hour --\0Cancel\0-5\0+5\0Okay\0Hour";
const char str_edit_minute[] PROGMEM = "-- Minute --\0Cancel\0-10\0+10\0Okay\0Min";
const char str_edit_hatch_control_hour[] PROGMEM = "- Hatch Hour -\0Cancel\0-5\0+5\0Okay\0Hatch Hour [0-23]";
const char str_edit_hatch_control_min[] PROGMEM = "- Hatch Min -\0Cancel\0-10\0+10\0Okay\0Hatch Min [0-59]";
const char str_edit_hatch_control_dur[] PROGMEM = "Hatch Duration\0Cancel\0-30\0+30\0Okay\0Duration Sec [5-900]";
const char str_edit_led_tw_on_hour[] PROGMEM = "LED Time Window\0Cancel\0-5\0+5\0Okay\0On Hour [12-21]";
const char str_edit_led_tw_on_min[] PROGMEM = "LED Time Window\0Cancel\0-10\0+10\0Okay\0On Min [0-59]";
const char str_edit_led_tw_off_hour[] PROGMEM = "LED Time Window\0Cancel\0-5\0+5\0Okay\0Off Hour [16-23]";
const char str_edit_led_tw_off_min[] PROGMEM = "LED Time Window\0Cancel\0-10\0+10\0Okay\0Off Min [0-59]";
const char str_edit_led_dur[] PROGMEM = "LED Duration\0Cancel\0-30\0+30\0Okay\0Duration Min [30-240]";
const char str_edit_led_max[] PROGMEM = "LED Max\0Cancel\0-5\0+5\0Okay\0Maximum [5-30]";
const char str_edit_amb_light_thr[] PROGMEM = "Light Threshold\0Cancel\0-50\0+50\0Okay\0Threshold [100-900]";
const char str_edit_amb_light_dur[] PROGMEM = "Avg Duration\0Cancel\0\0\0Okay\0Duration min [2-20]";

// setup header strings
const char str_show_parms_display[] PROGMEM = "Display";
const char str_show_parms_date_time[] PROGMEM = "Date & Time";
const char str_show_parms_batt_threshold[] PROGMEM = "Batt Thresh.";
const char str_show_parms_led_control[] PROGMEM = "LED Control";
const char str_show_parms_amb_light[] PROGMEM = "Amb Light";
const char str_show_parms_mp3[] PROGMEM = "MP3 Control";
const char str_show_parms_hatch[] PROGMEM = "Hatch Control";

// messages
const char msg_datetime_set[] PROGMEM = "Date and time set!";
const char msg_display_set[] PROGMEM = "Display parms set!";
const char msg_hatch_control_set[] PROGMEM = "Hatch parms set!";
const char msg_led_tw_set[] PROGMEM = "LED time window set!";
const char msg_led_dur_max_set[] PROGMEM = "LED dur/max set!";
const char msg_amb_light_set[] PROGMEM = "Amb Light parms set!";
const char msg_cancelled[] PROGMEM = "cancelled";

// startup messages
const char str_init_master[] PROGMEM = "System Startup -------";
const char str_init_version[] PROGMEM = "Version ";
const char str_init_keyboard[] PROGMEM = "Keyboard ...";
const char str_init_bat_status[] PROGMEM = "Battery ...";
const char str_init_amb_light_status[] PROGMEM ="Light sensors ...";
const char str_init_LED_status[] PROGMEM ="LED ...";
const char str_init_temp_sensors[] PROGMEM = "Temp sensors ...";
const char str_init_RTC[] PROGMEM = "RTC ...";
const char str_init_mp3_status[] PROGMEM = "MP3 ...";
const char str_init_hatch_control[] PROGMEM = "Hatch ...";

// vocabulary
const char str_voc_temp_missing[] PROGMEM = " --.-";
const char str_voc_duration[] PROGMEM = "Duration ";

// error messages
const char err_ext_temp_sensor[] PROGMEM = "err: external temp sensor\n";
const char err_int_temp_sensor[] PROGMEM = "err: internal temp sensor\n";

// title string
const char tit_temp_log_ext[] PROGMEM = "External temperature log";
const char tit_temp_log_int[] PROGMEM = "Internal temperature log";
const char tit_batt_log[] PROGMEM = "Battery voltage log";
const char tit_start_time_log[] PROGMEM = "Light Start Time";

// table for light levels, translates 0-10 to actual PWM values of 0-255
const uint8_t LED_light_table[MAX_LED_LEVEL + 1] PROGMEM = {0, 1, 2, 3, 5, 8, 12, 15, 20, 25, 30,
	36, 43, 50, 58, 66, 75, 84, 94, 104, 115, 127, 139, 152, 165, 178, 193, 208, 223, 239, 255};

